/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFM_ACTIONPLUGIN_H
#define FLYFM_ACTIONPLUGIN_H

#include <flyfm/gui_export.h>

#include <QAction>

#include <flyfm/plugin.h>

namespace FlyFM
{

class Entry;
class EntryList;
class ActionPlugin;

//////////////////////////////////////////////////////////////////

class FLYFM_GUI_EXPORT PluginAction : public QAction
{
    Q_OBJECT
public:
    enum Location { ActionMenu, SendMenu, ToolBar, ToolsMenu };

    PluginAction(const QIcon &icon, const QString &text, Location location, ActionPlugin *parent);
    virtual ~PluginAction();

    Location location() const;
    virtual bool isAllowedFor(const EntryList &) = 0;
    virtual void call(const Entry &parentEntry, const EntryList &entries) = 0;

protected:
    Location m_location;
};

using PluginActionPtrList = QList<PluginAction *>;

//////////////////////////////////////////////////////////////////

class FLYFM_GUI_EXPORT ActionPlugin : public Plugin
{
public:
    ActionPlugin();
    virtual ~ActionPlugin();

    virtual PluginActionPtrList actions() = 0;
    virtual QObject *toQObject() = 0;
};

//////////////////////////////////////////////////////////////////

class FLYFM_GUI_EXPORT ActionPluginV2 : public Plugin
{
public:
    ActionPluginV2();
    virtual ~ActionPluginV2();

    virtual QList<QAction *> actions(const Entry &parent, const EntryList &entries, QWidget *owner, QWidget *parentWidget) = 0;
    virtual QList<QAction *> sendActions(const Entry &parent, const EntryList &entries, QWidget *owner, QWidget *parentWidget) = 0;
};

}

#define FlyFM_ActionPlugin_iid FLY_ORG_DOMAIN".fly-fm.ActionPlugin/1.0"
#define FlyFM_ActionPluginV2_iid FLY_ORG_DOMAIN".fly-fm.ActionPluginV2/1.0"

Q_DECLARE_INTERFACE(FlyFM::ActionPlugin, FlyFM_ActionPlugin_iid)
Q_DECLARE_INTERFACE(FlyFM::ActionPluginV2, FlyFM_ActionPluginV2_iid)

#endif // FLYFM_ACTIONPLUGIN_H
