/*

  Base library
  Application Info

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMAPPINFO_H
#define FLYFMAPPINFO_H

#include <flyfm/core_export.h>

#include <QMetaType>
#include <QString>
#include <QList>

class QDBusArgument;

namespace FlyFM
{
class AppInfo;
using AppInfoList = QList<AppInfo>;
}

QDBusArgument &operator<<(QDBusArgument &arg, const FlyFM::AppInfo &appInfo);
const QDBusArgument &operator>>(const QDBusArgument &arg, FlyFM::AppInfo &appInfo);

namespace FlyFM
{
class FLYFM_CORE_EXPORT AppInfo
{
public:
    enum Source { Desktop = 0, User = 1 };

    AppInfo();
    AppInfo(const AppInfo &appInfo);

    void setName(const QString &name);
    void setComment(const QString &comment);
    void setExec(const QString &exec);
    void setIcon(const QString &icon);
    void setSource(Source source = User, const QString &desktopSourceFile = QString());
    void setSourceType(Source source);
    void setTerminal(bool on);

    bool isFromStandardDirs() const;

    QString name() const;
    QString comment() const;
    QString exec() const;
    QString icon() const;
    QString desktopSourceFilePath() const;
    QString desktopSourceFileName() const;
    Source source() const;
    bool terminal() const;

    AppInfo &operator=(const AppInfo &appInfo);
    bool operator==(const AppInfo &appInfo) const;
    bool operator!=(const AppInfo &appInfo) const;
    bool isNull() const;

    static QString nameByExec(const QString &exec);
    static AppInfo appInfo(const QString &desktopEntryFile);

protected:
    QDBusArgument &toDBusArgument(QDBusArgument &arg) const;
    const QDBusArgument &fromDBusArgument(const QDBusArgument &arg);

    friend QDBusArgument &::operator<<(QDBusArgument &arg, const AppInfo &appInfo);
    friend const QDBusArgument &::operator>>(const QDBusArgument &arg, AppInfo &appInfo);

    QString m_name;
    QString m_comment;
    QString m_exec;
    QString m_icon;
    QString m_desktopSourceFile;
    Source m_source;
    bool m_terminal;
};
}

#if !defined(QT_NO_DEBUG_STREAM)
FLYFM_CORE_EXPORT QDebug operator<<(QDebug, const FlyFM::AppInfo &);
#endif

Q_DECLARE_METATYPE(FlyFM::AppInfo)
Q_DECLARE_METATYPE(FlyFM::AppInfoList)

#endif // FLYFMAPPINFO_H
