/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMCLIPBOARD_H
#define FLYFMCLIPBOARD_H

#include <flyfm/core_export.h>

#include <QStringList>

#include <flyfm/entrylist.h>

#include <set>

class QClipboard;
class QMimeData;

namespace FlyFM
{

class ClipboardProcessor;

class FLYFM_CORE_EXPORT Clipboard
{
public:
    enum Operation { None = 0, DragDrop = 1, Copy = 2, Move = 3 };

    static Clipboard *ptr();

    EntryList entries(Operation operation = DragDrop) const;
    void setEntries(const EntryList &entries, Operation operation = DragDrop);
    static Entry compatibleEntry(const Entry &entry);

    QMimeData *dragMimeData(const EntryList &entries) const;
    EntryList dropEntries(const QMimeData *data) const;

    void clear();

    std::set<QString> archiveMountPoints() const;

protected:
    Clipboard();
    Clipboard(const Clipboard &) = delete;
    Clipboard &operator=(const Clipboard &) = delete;

    void registerProcessor(ClipboardProcessor *processor);

    QClipboard *clipboard();
    const QClipboard *clipboard() const;

    static const QMimeData *clipboardMimeData(const QClipboard *clipboard);
    void setMimeTypesData(const QStringList &mimeTypes, const QByteArray &data, QMimeData *mimeData);
    QStringList actualMimeTypes(const QMimeData *mimeData) const;

    QList<QUrl> urlsFor(Operation operation, const QMimeData *mimeData) const;

    static Clipboard *m_ptr;

    QList<ClipboardProcessor *> m_processors;
    QStringList m_supportedMimeTypes;
    std::set<QString> m_archiveMountPoints;
};

}

#endif // FLYFMCLIPBOARD_H
