/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMCORE_H
#define FLYFMCORE_H

#include <flyfm/core_export.h>

#include <QMap>
#include <QStringList>
#include <QVariant>

class QWidget;

namespace FlyFM
{

namespace Core
{

enum HaltReason { Exited = 0, Interrupted = 1, Segfaulted = 2 };

using ReleaseFuncType = void(*)();
using HaltHandlerFuncType = void(*)(HaltReason);

FLYFM_CORE_EXPORT void moveForCenter(QWidget *);
FLYFM_CORE_EXPORT void moveForPresentation(QWidget *);

FLYFM_CORE_EXPORT QString md5HashForString(const QString &string);
FLYFM_CORE_EXPORT QString dataSizeAsString(long long size); // Convert byte size to Kb, Mb, Gb
FLYFM_CORE_EXPORT QString version();
QString versionedName();

void registerReleaseFunction(ReleaseFuncType);
void unregisterReleaseFunction(ReleaseFuncType);

void registerHaltHandlerFunction(HaltHandlerFuncType);
void unregisterHaltHandlerFunction(HaltHandlerFuncType);

int compareVariants(const QVariant &v1, const QVariant &v2);

FLYFM_CORE_EXPORT bool startProcess(const QString &command);
FLYFM_CORE_EXPORT bool startProcess(const QString &program, const QStringList &arguments, const QString &workingDirectory = QString());

FLYFM_CORE_EXPORT bool shouldLoadPlugin(const QJsonObject &metaData);

FLYFM_CORE_EXPORT QMap<gid_t, QString> currentProcessGroups();

}

}

#endif // FLYFMCORE_H
