#ifndef FLYFMDESKTOPVIEW_H
#define FLYFMDESKTOPVIEW_H

#include <QListView>
#include <QImage>

namespace FlyFM
{

class Entry;
class EntryList;
class ViewSignalsObject;

class DesktopView : public QListView
{
    Q_OBJECT
public:
    DesktopView(ViewSignalsObject *signalsObject, QWidget *parent = nullptr);
    ~DesktopView();

    virtual EntryList selectedEntries();

public slots:
    void reset() override;

protected slots:
    void openActionEmitter();

    void customPopupContextMenu(const QPoint &);
    void possibleEntryPosition(const EntryList &, const QPoint &);
    void layoutChanged();

protected:
    void initializeBackground();
    void clearItemPositions();
    void pushItemPositions();
    void popItemPositions();

    void dropEvent(QDropEvent *event) override;
    void paintEvent(QPaintEvent *event) override;
    void mousePressEvent(QMouseEvent *event) override;
    void dragMoveEvent(QDragMoveEvent *) override;
    void wheelEvent(QWheelEvent *) override;

    QModelIndex m_overIndex;
    QImage m_desktopImage;
    ViewSignalsObject *m_signalsObject;

    friend class DesktopWidget;
    friend class PopupMenu;
};
}

#endif // FLYFMDESKTOPVIEW_H
