/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMDIRDIALOG_H
#define FLYFMDIRDIALOG_H

#include <flyfm/dialog.h>
#include <flyfm/vfs.h>
#include <flyfm/entrylist.h>

class QLabel;
class QLineEdit;
class QToolButton;

namespace FlyFM
{
class TreeWidget;
class Navigation;

class DirDialog : public Dialog
{
    Q_OBJECT
public:
    explicit DirDialog(QWidget *parent = nullptr, const QString &title = tr("Choose dir"), const Entry &root = VFS::ptr()->favouritesEntry());
    virtual ~DirDialog();

    EntryList selectedEntries() const;
    Entry selectedEntry() const;

    FLYFM_GUI_EXPORT static Entry getExistingDirectory(QWidget *parent = nullptr, const QString &title = QString(), const Entry &rootEntry = Entry());

    void setRootEntry(const Entry &root);
    Entry rootEntry() const;

    void setReadOnly(bool on);

protected:
    void changeEvent(QEvent *event) override;

signals:
    void aboutToAccept();

private slots:
    void accept() override;
    void entriesSelected(const EntryList &);

    void checkTabletMode();

private:
    void saveSettings() const;
    void loadSettings();

    virtual bool isProperEntryType(const Entry &);
    virtual bool canAcceptSelectedEntries();
    TreeWidget *m_pTreeWidget;
    EntryList m_selectedEntries;
    Navigation *m_navigation;

    QLabel *m_pLPath;
    QLineEdit *m_pLEPath;
    QToolButton *m_createFolderButton;

    bool m_isInTabletMode;
    QByteArray m_treeCommonState;
    QByteArray m_treeTabletState;
};
}

#endif // FLYFMDIRDIALOG_H
