/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMENTRY_H
#define FLYFMENTRY_H

#define UNDEFINED_ENTRY_TYPE        0

/* places 1000 - 10000 */

#define MIN_PLACE_INDEX         1000
#define MAX_PLACE_INDEX         9999

#define MIN_PURE_PLACE_INDEX        1000
#define MAX_PURE_PLACE_INDEX        1999

#define CUSTOM_PURE_PLACE       1000
#define COMPUTER_PLACE          1001
#define NETWORK_PLACE           1002
#define STORAGE_DEVICES_PLACE       1003
#define FAVOURITES_PLACE        1004
#define SAMBA_NETWORK_PLACE     1005
#define SAMBA_WORKGROUP_PLACE       1006
#define SAMBA_SERVER_PLACE      1007
#define FTP_NETWORK_PLACE       1008
#define SFTP_NETWORK_PLACE      1009
#define SMB_NETWORK_PLACE       1010

/* dir places 2000 - 2099 */

#define MIN_DIR_PLACE_INDEX     2000
#define MAX_DIR_PLACE_INDEX     2999

#define FILE_SYSTEM_ROOT_PLACE      2000
#define HOME_DIR_PLACE          2001
#define TRASH_DIR_PLACE         2002
#define DESKTOP_DIR_PLACE       2003
#define MY_DOCUMENTS_DIR_PLACE      2004
#define FAVOURITE_PLACE         2005

/* implicit dir places */

#define MIN_IMPLICIT_DIR_PLACE_INDEX    3500
#define MAX_IMPLICIT_DIR_PLACE_INDEX    4499

/* storage places 3000 - 3999 */

#define MIN_STORAGE_PLACE_INDEX     3500
#define MAX_STORAGE_PLACE_INDEX     3999

#define USB_DISK_STORAGE_PLACE      3500
#define CDROM_DRIVE_STORAGE_PLACE   3501
#define FLOPPY_DRIVE_STORAGE_PLACE  3502
#define HARD_DISK_DRIVE_STORAGE_PLACE   3503
#define SD_MMC_STORAGE_PLACE        3504
#define MEMORY_STICK_STORAGE_PLACE  3505
#define COMPACT_FLASH_STORAGE_PLACE 3506
#define SMART_MEDIA_STORAGE_PLACE   3507
#define MTP_STORAGE_PLACE           3508

/* other implicit dir places */

#define FAVOURITE_NETWORK_PLACE     4000
#define SAMBA_SHARE_PLACE       4001
#define FTP_SHARE_PLACE         4002
#define SFTP_SHARE_PLACE        4003
#define SMB_SERVER_PLACE        4004

/* regular file system objects 10000 - 11000 */

#define FILE_ENTRY_TYPE         10000
#define DIR_ENTRY_TYPE          10001
#define SYMLINK_ENTRY_TYPE      10002
#define FILE_LINK_ENTRY_TYPE    10003
#define DIR_LINK_ENTRY_TYPE     10004
#define ARCHIVE_ENTRY_TYPE      10005
#define ARCHIVE_LINK_ENTRY_TYPE 10006

#define ARCHIVE_FILE_MOUNT_POINT    20000
#define CUSTOM_URL          30000
#define FUSE_URL            30100

#include <flyfm/vfs_export.h>

#include <QMetaType>
#include <QString>
#include <QHash>

class QDBusArgument;
class QDataStream;

namespace FlyFM
{
class Entry;
}

QDBusArgument &operator<<(QDBusArgument &arg, const FlyFM::Entry &entry);
const QDBusArgument &operator>>(const QDBusArgument &arg, FlyFM::Entry &entry);

QDataStream &operator<<(QDataStream &out, const FlyFM::Entry &entry);
QDataStream &operator>>(QDataStream &in, FlyFM::Entry &entry);

namespace FlyFM
{

class FLYFM_VFS_EXPORT Entry
{
public:
    Entry();

    QString path() const;
    QString localPath() const;
    QString localPathWithScheme() const;
    QString archiveMountPointOrLocalPath() const;

    bool isFile() const;
    bool isDir() const;
    bool isFileLink() const;
    bool isDirLink() const;
    bool isPlace() const;
    bool isCustomUrl() const;
    bool isFuseUrl() const;
    bool isExists() const;

    bool isPurePlace() const;
    bool isComputerPlace() const;
    bool isFavouritesPlace() const;
    bool isNetworkPlace() const;

    bool isSambaNetworkPlace() const;
    bool isSambaWorkgroupPlace() const;
    bool isSambaServerPlace() const;
    bool isSambaSharePlace() const;
    bool isSambaPlace() const;

    bool isSmbNetworkPlace() const;
    bool isFtpNetworkPlace() const;
    bool isSFtpNetworkPlace() const;
    bool isFtpSharePlace() const;
    bool isSFtpSharePlace() const;
    bool isSmbServerPlace() const;

    bool isFavouriteNetworkPlace() const;

    bool isDirPlace() const;
    bool isImplicitDirPlace() const;
    bool isStoragePlace() const;

    bool isFavouritePlace() const;

    bool isFileSystemRootPlace() const;
    bool isHomeDirPlace() const;
    bool isTrashDirPlace() const;
    bool isDesktopDirPlace() const;
    bool isMyDocumentsDirPlace() const;
    bool isStorageDevicesPlace() const;

    bool isUsbDiskStoragePlace() const;
    bool isCdromDriveStoragePlace() const;
    bool isFloppyDriveStoragePlace() const;
    bool isHardDiskDriveStoragePlace() const;

    bool isArchiveMountPoint() const;
    bool isInArchive() const;
    bool isSimilar(const Entry &entry) const;
    bool isPlaceBased() const;

    int type() const;
    bool isNull() const;

    bool operator<(const Entry &entry) const;
    bool operator>(const Entry &entry) const;
    bool operator==(const Entry &entry) const;
    bool operator!=(const Entry &entry) const;

    QString lastPathSection() const;

    QString absoluteFilePath() const;
    QString fileName() const;
    bool exists() const;
    bool isReadable() const;
    bool isWritable() const;
    bool isExecutable() const;
    bool isSymLink() const;
    bool isBrokenSymlink() const;
    QString symLinkTarget() const;
    bool isRoot() const;
    bool isFSRoot() const;
    bool isHidden() const;
    struct CreatedInfo
    {
        time_t time;
        bool valid;
    };
    CreatedInfo created() const;
    time_t lastModified() const;
    time_t lastStatusChange() const;
    qint64 fileSize() const;
    QString owner() const;
    uint ownerId() const;
    QString group() const;
    uint groupId() const;

    static QString userName(uid_t user);
    static QString groupName(gid_t group);

protected:
    QDBusArgument &toDBusArgument(QDBusArgument &arg) const;
    const QDBusArgument &fromDBusArgument(const QDBusArgument &arg);

    QDataStream &toDataStream(QDataStream &out) const;
    QDataStream &fromDataStream(QDataStream &in);

    QString m_path;
    QString m_localPath;

    int m_type;

    friend class VFS;
    friend class VFSPrivate;
    friend class VFSPlaceBase;
    friend class VFSService;

    friend QDBusArgument &::operator<<(QDBusArgument &arg, const Entry &entry);
    friend const QDBusArgument &::operator>>(const QDBusArgument &arg, Entry &entry);

    friend QDataStream &::operator<<(QDataStream &out, const Entry &entry);
    friend QDataStream &::operator>>(QDataStream &in, Entry &entry);
};

inline uint qHash(const Entry &entry)
{
    return qHash(entry.path());
}

}

#if !defined(QT_NO_DEBUG_STREAM)
FLYFM_VFS_EXPORT QDebug operator<<(QDebug, const FlyFM::Entry &);
#endif

Q_DECLARE_METATYPE(FlyFM::Entry)

#endif // FLYFMENTRY_H
