#ifndef ENTRYVIEWPARAMETERS_H
#define ENTRYVIEWPARAMETERS_H

#include <flyfm/core_export.h>

#include <QObject>

namespace FlyFM
{

class FLYFM_CORE_EXPORT EntryViewParameters : public QObject
{
    Q_OBJECT
public:
    static EntryViewParameters *ptr();

    void setUseMetaInfoForDirNames(bool enable);
    bool useMetaInfoForDirNames() const;

    void setUseMetaInfoForDirIcons(bool enable);
    bool useMetaInfoForDirIcons() const;

    void setShowThumbnails(bool enable);
    bool showThumbnails() const;

    void setShowThumbnailsInNetwork(bool enable);
    bool showThumbnailsInNetwork() const;

    void setShownThumbnailGroups(const QStringList &groups);
    QStringList shownThumbnailGroups() const;

    void setMaxImageFileSizeToShowThumbnailMB(int size);
    int maxImageFileSizeToShowThumbnailMB() const;

    void setNumericSortingEnabled(bool on);
    bool isNumericSortingEnabled() const;

    void setLocaleSortingEnabled(bool on);
    bool isLocaleSortingEnabled() const;

    void setShowFileSizeInBytes(bool on);
    bool showFileSizeInBytes() const;

    void setKineticScrollingEnabled(bool on);
    bool isKineticScrollingEnabled() const;

    void setPinchZoomEnabled(bool on);
    bool isPinchZoomEnabled() const;

signals:
    void useMetaInfoForDirsChanged();
    void showThumbnailsChanged();
    void showThumbnailsInNetworkChanged();
    void shownThumbnailGroupsChanged();
    void maxImageFileSizeToShowThumbnailChanged();
    void numericSortingEnabledChnaged();
    void localeSortingEnabledChnaged();
    void showFileSizeInBytesChanged();

private:
    EntryViewParameters(QObject *parent = nullptr);

    bool m_useMetaInfoForDirNames;
    bool m_useMetaInfoForDirIcons;
    bool m_showThumbnails;
    bool m_showThumbnailsInNetwork;
    QStringList m_shownThumbnailGroups;
    int m_maxImageFileSizeToShowThumbnailMB;
    bool m_numericSorting;
    bool m_localeSorting;
    bool m_showFileSizeInBytes;
    bool m_kineticScrollingEnabled;
    bool m_pinchZoomEnabled;
};

} // namespace FlyFM

#endif // ENTRYVIEWPARAMETERS_H
