/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMENTRYWATCHER_H
#define FLYFMENTRYWATCHER_H

#include <flyfm/vfs_export.h>
#include <flyfm/vfs.h>

#include <QObject>
#include <QStringList>
#include <QMap>
#include <QSet>
#include <QReadWriteLock>

namespace FlyFM
{

class Entry;
class EntryList;
class InotifyWatcher;

class FLYFM_VFS_EXPORT EntryWatcher : public QObject
{
    Q_OBJECT
public:
    EntryWatcher(SmbDirCache *smbDirCache, QObject *parent);
    virtual ~EntryWatcher();

    void addEntry(const Entry &entry);
    void removeEntry(const Entry &entry);
    void setDisabled(bool disabled);
    void clear();

    void addBadEntry(const Entry &entry);
    void listDir(const Entry &dirEntry, VFS::EntryChildFilters filters);

    void notifyEntryCreated(const Entry &entry);
    void notifyEntryDeleted(const Entry &entry);
    void notifyEntryChanged(const Entry &entry);

signals:
    void entryCreated(const Entry &entry);
    void entryDeleted(const Entry &entry);
    void entryChanged(const Entry &entry);
    void entryAttributesChanged(const Entry &entry);
    void entryReset(const Entry &entry);
    void dirListed(const Entry &dirEntry, VFS::EntryChildFilters filters, const EntryList &entries);

protected slots:
    void entryChangedNotify(const Entry &entry, bool anyWay = false);

    void unregisterPath(const QString &path);

    void slotPathCreated(const QString &path, const QString &fileName);
    void slotPathDeleted(const QString &path, const QString &fileName);
    void slotPathChanged(const QString &path, const QString &fileName);
    void slotPathAttributesChanged(const QString &path, const QString &fileName);

protected:
    using PathSignal = void (EntryWatcher::*)(const Entry &);
    void notifyPath(PathSignal signal, const QString &path, const QString &fileName);
    bool isRefreshable(const QString &localPath) const;

    InotifyWatcher *inotifyWatcher;
    QReadWriteLock m_lock;
    QMap<QString, QSet<Entry>> m_pathToPlaceRelatedPathsMap;
    bool m_isDisabled;
    QStringList m_unrefreshableRoots;
    struct ListDirRequest;
    std::vector<ListDirRequest *> m_listDirRequests;

    friend class VFS;
};
}

#endif // FLYFMENTRYWATCHER_H
