/*

  Graphic user's interface library
  File dialog

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMFILEDIALOG_H
#define FLYFMFILEDIALOG_H

#include <flyfm/dialog.h>
#include <flyfm/entry.h>
#include <flyfm/vfs.h>
#include <flyfm/model.h>

class QHBoxLayout;
class QGridLayout;
class QLabel;
class QToolBar;
class QPushButton;
class QAction;
class QComboBox;
class QCheckBox;
class QLineEdit;
class QDialogButtonBox;
class QSplitter;

namespace FlyFM
{

class NavigationActions;
class RefreshAction;
class LocationBar;
class FolderWidget;
class PlacesWidget;

class FileDialog : public Dialog
{
    Q_OBJECT
public:
    enum Type { OpenDialog = 0, SaveDialog = 1 };

    explicit FileDialog(QWidget *parent = nullptr, const QString &title = QString(), const Entry &root = Entry(), Type type = OpenDialog, const QStringList &nameFilters = QStringList(), ModelFilter::EntryFilters entryFilters = ModelFilter::AllEntries);
    virtual ~FileDialog();

    void setNameFilters(const QStringList &filters);
    void setNameFilter(const QString &filter);
    void selectNameFilter(const QString &filter);
    QString nameFilter() const;

    EntryList selectedEntries() const;
    Entry selectedEntry() const;

    void selectEntry(const Entry &entry);

    void setEntryFilters(ModelFilter::EntryFilters filters);

    void setType(Type type);
    Type type() const;

    void setDefaultSuffix(const QString &suffix);
    QString defaultSuffix() const;

    void setViewMode(Model::Mode mode);
    Model::Mode viewMode() const;

    enum FileMode { AnyFile, ExistingFile, ExistingFiles };

    void setFileMode(FileMode mode);
    FileMode fileMode() const;

    void setConfirmOverwrite(bool enabled);
    bool confirmOverwrite() const;

    void setRootEntry(const Entry &root);
    Entry rootEntry() const;

    void setExtraEntries(const EntryList &entries);
    EntryList extraEntries() const;

    static Entry getSaveEntry(QWidget *parent = nullptr, const QString &title = tr("Save"), const Entry &root = FlyFM::VFS::ptr()->computerEntry(), const QStringList &nameFilters = QStringList("*"), ModelFilter::EntryFilters entryFilters = ModelFilter::AllEntries);
    FLYFM_GUI_EXPORT static Entry getOpenEntry(QWidget *parent = nullptr, const QString &title = tr("Open"), const Entry &root = FlyFM::VFS::ptr()->computerEntry(), const QStringList &nameFilters = QStringList("*"), ModelFilter::EntryFilters entryFilters = ModelFilter::AllEntries);
    static EntryList getOpenEntries(QWidget *parent = nullptr, const QString &title = tr("Open"), const Entry &root = FlyFM::VFS::ptr()->computerEntry(), const QStringList &nameFilters = QStringList("*"), ModelFilter::EntryFilters entryFilters = ModelFilter::AllEntries);

    QGridLayout *extraControlsLayout();

public slots:
    int exec() override;

signals:
    void filterSelected(const QString &filter);
    void rootEntryChanged(const Entry &entry);
    void entrySelected(const Entry &entry);
    void entriesSelected(const EntryList &entries);
    void entryHighlighted(const Entry &entry);
    void aboutToAccept();

protected:
    void changeEvent(QEvent *event) override;

private slots:
    void accept() override;

    void filterChanged(const QString &);
    void slotEntriesSelected();
    //void entrySelected(const Entry &);
    void updatePreview();
    void autoCompleteFileName(const QString &);

    void checkTabletMode();

private:
    bool canAcceptSelectedEntries();
    static QStringList nameFiltersForModel(const QString &nameFilter);
    static QString nameFilterFromModel(const QStringList &nameFilters);
    EntryList typedEntries() const;
    QStringList addSuffixToFiles(const QStringList &files) const;

    void saveSettings() const;
    void loadSettings();

    Type m_type;

    QLabel *m_pLLookAt;
    QToolBar *m_pToolBar;
    QPushButton *m_pGoButton;
    QAction *m_previewModeAction;

    QHBoxLayout *m_filterLayout;
    QLabel *m_filterLabel;
    QComboBox *m_filterComboBox;

    NavigationActions *m_pNavigationActions;
    RefreshAction *m_pRefreshAction;
    LocationBar *m_pLocationBar;

    QHBoxLayout *m_middleLayout;
    QSplitter *m_pSplitter;
    PlacesWidget *m_pPlacesWidget;
    FolderWidget *m_pFolderWidget;
    QLabel *m_pPreviewLabel;

    QGridLayout *m_bottomItemLayout;
    QLabel *m_pLPath;
    QLineEdit *m_pLEPath;

    QString m_defaultSuffix;
    FileMode m_fileMode;
    bool m_confirmOverwrite;

    QDialogButtonBox *m_buttonBox;

    bool m_isInTabletMode;
    QByteArray m_placesCommonState;
    QByteArray m_placesTabletState;
    QByteArray m_folderCommonState;
    QByteArray m_folderTabletState;
};

}

#endif // FLYFMFILEDIALOG_H
