/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMFOPS_H
#define FLYFMFOPS_H

#include <flyfm/core_export.h>
#include <flyfm/entrylist.h>

#include <QMutex>
#include <QMap>

#include <flyfm/entry.h>
#include <flyfm/serviceinterfacesingletone.h>

namespace FlyFM
{

class FLYFM_CORE_EXPORT FOpId
{
public:
    FOpId();
    FOpId(const FOpId &id);
    ~FOpId();

    bool isNull() const;

    FOpId &operator=(const FOpId &id);
    bool operator<(const FOpId &id) const;
    bool operator==(const FOpId &id) const;

private:
    FOpId(const QString &op);
    void setStamp(const QString &);
    QString stamp() const;

    QString m_stamp;

    friend class FOps;
};

class FLYFM_CORE_EXPORT FOps : public QObject, public ServiceInterfaceSingletone
{
    Q_OBJECT
public:
    static FOps *ptr();
    ~FOps();

    FOpId makeDir(const Entry &entry);
    FOpId makeFile(const Entry &entry);
    FOpId makeShortcut(const Entry &entry);
    QStringList fileTemplates() const;
    FOpId makeFromTemplate(const Entry &entry, const QString &templatePath);
    FOpId makeShortcuts(const EntryList &entryList, const Entry &dest = Entry());
    FOpId makeTitledShortcut(const QString &url, const QString &title, const Entry &dest);
    FOpId copy(const EntryList &srcs, const Entry &dest);
    FOpId rename(const Entry &source, const Entry &destination);
    FOpId renameForUndo(const Entry &source, const Entry &destination);
    FOpId move(const EntryList &srcs, const Entry &dest);
    FOpId moveForUndo(const EntryList &srcs, const Entry &dest);
    FOpId remove(const EntryList &entries, bool silent = false);
    FOpId trash(const EntryList &entries, bool silent = false);
//      FOpId restore( const EntryList &entries );
    FOpId restore(const Entry &entry);
    FOpId cleanTrash();

    int operationProgress(const FOpId &opId) const;

    static bool isTrash(const Entry &entry);
    static bool isTrashEntries(const EntryList &entries);
    static bool isEmptyTrash();

    enum OperationType {
        Trash,
        Create,
        Copy,
        Move,
        Rename
    };

signals:
    void operationStarted(OperationType type, const FlyFM::FOpId &);
    void operationFinished(const FlyFM::FOpId &, int, const FlyFM::EntryList &);

protected slots:
    void finished(const QString &, int, const FlyFM::EntryList &);

protected:
    FOps(QObject *parent = nullptr);

private:
    static FOps *m_ptr;

    QWeakPointer<QDBusInterface> m_interface;
};

}

#endif // FLYFMFOPS_H
