#ifndef FLYFM_INOTIFYWATCHER_H
#define FLYFM_INOTIFYWATCHER_H

#include <QObject>
#include <QMap>
#include <QSet>
#include <QReadWriteLock>

class QSocketNotifier;

namespace FlyFM
{

class InotifyWatcher : public QObject
{
    Q_OBJECT
public:
    explicit InotifyWatcher(QObject *parent = nullptr);
    ~InotifyWatcher();

    bool addPath(const QString &path);
    bool removePath(const QString &path);

    bool isPathRegistered(const QString &path) const;

    bool clear();

signals:
    void pathUnregistered(const QString &path);

    void pathCreated(const QString &path, const QString &fileName);
    void pathDeleted(const QString &path, const QString &fileName);
    void pathChanged(const QString &path, const QString &fileName);
    void pathAttributesChanged(const QString &path, const QString &fileName);

private slots:
    void processInotifyMessages();
    void notifyDeletedSymlinks();

private:
    using PathSignal = void (InotifyWatcher::*)(const QString &path, const QString &fileName);
    void notifySymlinks(PathSignal signal, const QString &targetPath, const QString &fileName);

private:
    bool removePath(const QString &path, bool removeWatch);
    bool isSymlinkRegistered(const QString &path) const;

private:
    int m_ifd;
    QSocketNotifier *m_notifier;
    QMap<QString, int> m_pathToDescriptorMap;
    QMap<QString, QSet<QString>> m_symlinkTargetToSourcesMap;
    QSet<QString> m_deletedSymlinks;
    mutable QReadWriteLock m_lock;
};

} // namespace FlyFM

#endif // FLYFM_INOTIFYWATCHER_H
