#ifndef FLYFM_JOB_H
#define FLYFM_JOB_H

#include <flyfm/vfs_export.h>

#include <QObject>

namespace FlyFM
{

class JobPrivate;

class FLYFM_VFS_EXPORT Job : public QObject
{
    Q_OBJECT
public:
    explicit Job(QObject *parent = nullptr);
    ~Job();

    virtual void start() = 0;

    bool isDeleteOnFinish() const;
    void setDeleteOnFinish(bool on);

    const QVariant data() const;
    void setData(const QVariant &data);

    void exec();

    int error() const;
    QString errorString() const;

    void setError(int errorCode, const QString &errorString);

public slots:
    void finishedOkLater();
    void finishedOk();
    void finishedWithError(int errorCode, const QString &errorString);

signals:
    void finished(Job *);

protected:
    QScopedPointer<JobPrivate> d_ptr;
    Job(JobPrivate &dd, QObject *parent = nullptr);

private:
    Q_DECLARE_PRIVATE(Job)
};

} // namespace FlyFM

#endif // FLYFM_JOB_H
