#ifndef FLYFM_MODELFILTER_H
#define FLYFM_MODELFILTER_H

#include <flyfm/core_export.h>

#include <QFlags>
#include <QScopedPointer>

class QStringList;

template <class T> class QList;

namespace FlyFM
{

class Entry;
class EntryMatchValidator;
class PropertyEnumerator;

class ModelFilterPrivate;

class FLYFM_CORE_EXPORT ModelFilter
{
public:
    enum EntryFilter {
        UnspecFilter = 0,
        DirEntries = 1,
        ExecutableEntries = 2,
        AllEntries = 4,
        HiddenEntries = 8
    };

    Q_DECLARE_FLAGS(EntryFilters, EntryFilter)

    ModelFilter();
    virtual ~ModelFilter();

    bool isAllowedEntry(const Entry &entry) const;

    void setPropertyEnumerator(PropertyEnumerator *propertyEnumerator);

    void setEntryFilters(EntryFilters entryFilters);
    EntryFilters entryFilters() const;

    bool showHiddenEntries() const;
    void setShowHiddenEntries(bool show);

    void setNameFilters(const QStringList &nameFilters);
    QStringList nameFilters() const;

    void setNameFiltersForFilesOnly(bool on);
    bool nameFiltersForFilesOnly() const;

    void setValidators(const QList<EntryMatchValidator *> &validators);

    bool flushChanges();

private:
    QScopedPointer<ModelFilterPrivate> d;

    Q_DISABLE_COPY(ModelFilter)
};

Q_DECLARE_OPERATORS_FOR_FLAGS(ModelFilter::EntryFilters)

} // namespace FlyFM

#endif // FLYFM_MODELFILTER_H
