/*

  Explorer Graphic User's Interface Library
  File System Model Item

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFM_MODELITEM_H
#define FLYFM_MODELITEM_H

#include <flyfm/core_export.h>
#include <flyfm/entry.h>

namespace FlyFM
{

class Model;
class PropertyDataHolder;
class PropertyEnumerator;

class FLYFM_CORE_EXPORT ModelItem
{
public:
    ModelItem(const Entry &entry, const Model *model, ModelItem *parent = nullptr);

    QVariant data(int role, int col = 0) const;

    bool needsRefine() const;
    void delayedRefine();
    void refine();

    void refresh();
    void updatePresentation();

    void refinePropertyAtIndex(int index);

    static QVariant headerData(const Model *model, int role, int col = 0);
    static int columnCount(const Model *model);

    ModelItem *parent() const;
    ModelItem *child(int row) const;
    QList<ModelItem *> childs() const;
    ModelItem *child(const Entry &entry) const;
    const ModelItem *find(const Entry &entry) const;
    QList<ModelItem *> finds(const Entry &entry) const;

    Entry entry() const;

    uint childCount() const;
    uint row() const;

    void setChildsInited(bool flag);
    bool isChildsInited() const;

    void setChildsIniting(bool flag);
    bool isChildsIniting() const;

    bool hasChildItem(const Entry &entry) const;
    //bool hasChilds();
    bool hasNodeChilds() const;

    bool isDotDotDir() const;
    void setDotDotDir(bool on);

private:
    ~ModelItem();

protected:
    void init();

    QString description() const;
    QList<int> exploreIndexPath() const;

    QList<ModelItem *>::const_iterator lowerBoundChildItem(ModelItem *childItem) const;
    QList<ModelItem *>::const_iterator upperBoundChildItem(ModelItem *childItem, QList<ModelItem *>::const_iterator startIt) const;
    int estimatedChildIndex(ModelItem *childItem) const;
    void insertChild(int index, ModelItem *child);
    void removeChild(ModelItem *child);

    void resort();

    PropertyEnumerator *propertyEnumerator();
    const PropertyEnumerator *propertyEnumerator() const;

    Entry m_entry;
    const Model *m_model;
    ModelItem *m_parent;

    bool m_hasNodeChilds            : 1;
    bool m_isChildsInited           : 1;
    bool m_isChildsIniting          : 1;
    bool m_needsRefine              : 1;
    bool m_hasNodeChildsNeedsRefine : 1;
    bool m_isDotDotDir              : 1;

    PropertyDataHolder *m_dataHolder;

    QList<ModelItem *> m_propertyOrientedChildList;

    friend class Model;
    friend class ModelPrivate;
    friend class TreeModelPrivate;
    friend class PlacesModelPrivate;
};

}

#endif // FLYFM_MODELITEM_H
