#ifndef FLYFM_MOUNTFTPURLJOB_H
#define FLYFM_MOUNTFTPURLJOB_H

#include <QUrl>
#include <QSharedPointer>

#include <flyfm/job.h>

class QProcess;

namespace FlyFM
{

class Authenticator;

class MountFtpURLJob : public FlyFM::Job
{
    Q_OBJECT
public:
    enum Error { NoError = 0, NotSambaEntry, LoginCancelled, NoShareEntry, NotInFuseGroup, InternalError };

    explicit MountFtpURLJob(QSharedPointer<Authenticator> authenticator, const QString &url, QObject *parent = nullptr);
    ~MountFtpURLJob();

    void start() override;

    QString url() const;
    QString localPath() const;

private slots:
    void readStandardError();
    void processFinished(int res);

private:
    void run();
    static QString ftpDirPath();
    void updateMountPath();
    bool checkAlreadyMounted();

private:
    QUrl m_url;

    QString m_pathTail;
    QString m_localPath;

    QProcess *m_process;
    QString m_mountPath;

    bool m_accessDenied;
    bool m_failedToConnect;
    bool m_userChanged;
    bool m_utf8;
    bool m_utf8AtStart;

    bool m_terminated;

    QSharedPointer<Authenticator> m_authenticator;
};

} // namespace FlyFM

#endif // FLYFM_MOUNTFTPURLJOB_H
