#ifndef FLYFM_MOUNTSFTPURLJOB_H
#define FLYFM_MOUNTSFTPURLJOB_H

#include <QUrl>
#include <QSharedPointer>

#include <flyfm/job.h>

namespace FlySu
{
class PtyProcess;
}

namespace FlyFM
{

class Authenticator;

class MountSFtpURLJob : public FlyFM::Job
{
    Q_OBJECT
public:
    enum Error { NoError = 0, NotSambaEntry, LoginCancelled, NoShareEntry, NotInFuseGroup, InternalError };

    explicit MountSFtpURLJob(QSharedPointer<Authenticator> authenticator, const QString &url, QObject *parent = nullptr);
    ~MountSFtpURLJob();

    void start() override;

    QString url() const;
    QString localPath() const;

private:
    void run();
    static QString sftpDirPath();
    void updateMountPath();
    bool checkAlreadyMounted();

    void converseSshfs();
    void processFinished(int res);

private:
    QUrl m_url;

    QString m_pathTail;
    QString m_localPath;

    FlySu::PtyProcess *m_process;
    QString m_mountPath;

    bool m_accessDenied;
    bool m_failedToConnect;
    bool m_userChanged;

    bool m_terminated;

    QSharedPointer<Authenticator> m_authenticator;
};

} // namespace FlyFM

#endif // FLYFM_MOUNTSFTPURLJOB_H
