/*

  Graphic user's interface library
  Navigation control

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMNAVIGATION_H
#define FLYFMNAVIGATION_H

#include <flyfm/gui_export.h>

#include <QObject>

#include <flyfm/entrylist.h>

class QMouseEvent;

namespace FlyFM
{

class Job;

class FLYFM_GUI_EXPORT Navigation : public QObject
{
    Q_OBJECT
public:
    Navigation(QWidget *parent = nullptr);
    virtual ~Navigation();

    Entry current() const;
    Entry previous() const;
    Entry validParent() const;
    void setLockedEntries(const EntryList &entries);

    EntryList backList() const;
    EntryList forwardList() const;

    bool isBackEnabled() const;
    bool isForwardEnabled() const;
    bool isUpEnabled() const;

    bool mouseButtonNavigate(QMouseEvent *e);

    Entry topLevelEntry() const;
    void setTopLevelEntry(const Entry &entry);

signals:
    void backEnabled(bool flag);
    void forwardEnabled(bool flag);
    void upEnabled(bool flag);

    void currentChanged(const Entry &entry);
    void backListUpdated(const EntryList &list);
    void forwardListUpdated(const EntryList &list);
    void topLevelEntryChanged();

    void highlightEntry(const Entry &entry);

public slots:
    bool setCurrent(const Entry &entry);

    virtual void back(int count = 1);
    virtual void forward(int count = 1);
    virtual void up();

protected:
    void emitEnableSignals();

private:
    bool isValidCurrentIndex() const;
    void setCurrentIndex(int index);
    void appendEntry(const Entry &entry);

private:
    int m_current;
    EntryList m_list;
    EntryList m_lockedEntries;
    QWidget *m_parentWidget;
    Entry m_topLevelEntry;
};
}

#endif // FLYFMNAVIGATION_H
