/*

  Graphic user's interface library
  Navigation action group

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMNAVIGATIONACTIONS_H
#define FLYFMNAVIGATIONACTIONS_H

#include <flyfm/gui_export.h>

#include <QObject>
#include <QMultiMap>

class QToolBar;
class QMenu;
class QAction;

namespace FlyFM
{

class Navigation;
class Entry;
class EntryList;

class FLYFM_GUI_EXPORT NavigationActions : public QObject
{
    Q_OBJECT
public:
    explicit NavigationActions(Navigation *pNavigation, QObject *parent = nullptr);
    ~NavigationActions();

    void addToToolBar(QToolBar *pToolBar);
    void addToMenu(QMenu *pMenu);
    QMenu *createMenu(QWidget *parent);
    QAction *upAction();

    void setNavigation(Navigation *navigation);

public slots:
    void updateBackMenu(const EntryList &list);
    void updateForwardMenu(const EntryList &list);

    void menuEntryChanged(const Entry &entry);

protected slots:
    void backMenuActionActivated(QAction *action);
    void forwardMenuActionActivated(QAction *action);

protected:
    QAction *m_pABack;
    QAction *m_pAForward;
    QAction *m_pAUp;

    QMenu *m_backMenu;
    QMultiMap<Entry, QAction *> m_backEntryActionMap;

    QMenu *m_forwardMenu;
    QMultiMap<Entry, QAction *> m_forwardEntryActionMap;

    Navigation *m_navigation;
};
}

#endif // FLYFMNAVIGATIONACTIONS_H
