#ifndef SMBPASSWORDSJOBS_H
#define SMBPASSWORDSJOBS_H

#include <flyfm/gui_export.h>

#include <QMap>
#include <QString>

#include <KJob>

#include <flysecretsserviceitem.h>

namespace FlyFM
{

class FLYFM_GUI_EXPORT FindNetPasswordJob : public KJob
{
    Q_OBJECT
public:
    FindNetPasswordJob(const QString &protocol, const QString &domain, const QString &server, const QString &object, QObject *parent = nullptr);

    void setUser(const QString &user);

    void start() override;

    using Item = QExplicitlySharedDataPointer<FlySecretsService::SecretItem>;
    using ItemList = QList<Item>;

    QMap<QString, QString> logins() const;

private slots:
    void slotSearchJobDone(KJob *job);

private:
    QHash<Item, QString> filterItems(const ItemList &items);

private:
    QString m_protocol;
    QString m_domain;
    QString m_server;
    QString m_object;
    QString m_user;

    QMap<QString, QString> m_logins;
};

class SetNetPasswordJob : public KJob
{
    Q_OBJECT
public:
    SetNetPasswordJob(const QString &protocol,
                      const QString &domain,
                      const QString &server,
                      const QString &object,
                      const QString &user,
                      const QString &password,
                      QObject *parent = nullptr);

    void start() override;

private slots:
    void slotCreateItemJobDone(KJob *job);

private:
    QString m_protocol;
    QString m_domain;
    QString m_server;
    QString m_object;
    QString m_user;
    QString m_password;
};

}

#endif // SMBPASSWORDSJOBS_H
