#ifndef FLYFM_NETWORKPLACESMANAGERPROXY_H
#define FLYFM_NETWORKPLACESMANAGERPROXY_H

#include <flyfm/vfs_export.h>

#include <QObject>
#include <QWeakPointer>
#include <QDBusInterface>

#include <flyfm/networkplacev2.h>

namespace FlyFM
{

class FLYFM_VFS_EXPORT NetworkPlacesManagerProxy : public QObject
{
    Q_OBJECT
public:
    explicit NetworkPlacesManagerProxy(QWeakPointer<QDBusInterface> interface, QObject *parent = nullptr);

    QList<NetworkPlaceV2> networkPlaces();
    bool addNetworkPlace(const NetworkPlaceV2 &place);
    void removeNetworkPlace(const NetworkPlaceV2 &place);
    void updateNetworkPlace(const NetworkPlaceV2 &place);
    int freeNetworkPlaceId();

signals:
    void networkPlaceAdded(const FlyFM::NetworkPlaceV2 &place);
    void networkPlaceRemoved(const FlyFM::NetworkPlaceV2 &place);
    void networkPlaceUpdated(const FlyFM::NetworkPlaceV2 &place);

private slots:
    void slotNetworkPlaceAdded(const FlyFM::NetworkPlaceV2 &place);
    void slotNetworkPlaceRemoved(const FlyFM::NetworkPlaceV2 &place);
    void slotNetworkPlaceUpdated(const FlyFM::NetworkPlaceV2 &place);

private:
    QWeakPointer<QDBusInterface> m_interface;

    QList<NetworkPlaceV2> m_places;
    bool m_cached;
};

} // namespace FlyFM

#endif // FLYFM_NETWORKPLACESMANAGERPROXY_H
