#ifndef FLYFM_NETWORKPLACEV2_H
#define FLYFM_NETWORKPLACEV2_H

#include <flyfm/vfs_export.h>

#include <QString>
#include <QList>
#include <QMetaType>

class QDataStream;
class QDBusArgument;

namespace FlyFM
{
class NetworkPlaceV2;
using NetworkPlaceListV2 = QList<NetworkPlaceV2>;
}

QDataStream &operator<<(QDataStream &out, const FlyFM::NetworkPlaceV2 &place);
QDataStream &operator>>(QDataStream &in, FlyFM::NetworkPlaceV2 &place);

QDBusArgument &operator<<(QDBusArgument &arg, const FlyFM::NetworkPlaceV2 &networkPlace);
const QDBusArgument &operator>>(const QDBusArgument &arg, FlyFM::NetworkPlaceV2 &networkPlace);

namespace FlyFM
{

class FLYFM_VFS_EXPORT NetworkPlaceV2
{
public:
    enum Flag {
        NoFlags = 0x0,
        Hidden = 0x2
    };
    Q_DECLARE_FLAGS(Flags, Flag)

    NetworkPlaceV2();
    NetworkPlaceV2(int id, const QString &name, const QString &url, const QString &localPath = QString());

    bool operator==(const NetworkPlaceV2 &place) const;

    bool isNull() const;

    int id() const;
    QString name() const;
    QString url() const;
    QString localPath() const;
    void setLocalPath(const QString &path);
    bool isSystem() const;
    bool isHidden() const;

    static int firstUserNetworkPlaceId();

private:
    friend QDataStream &::operator<<(QDataStream &out, const NetworkPlaceV2 &place);
    friend QDataStream &::operator>>(QDataStream &in, NetworkPlaceV2 &place);

    QDBusArgument &toDBusArgument(QDBusArgument &arg) const;
    const QDBusArgument &fromDBusArgument(const QDBusArgument &arg);

    friend QDBusArgument &::operator<<(QDBusArgument &arg, const NetworkPlaceV2 &networPlace);
    friend const QDBusArgument &::operator>>(const QDBusArgument &arg, NetworkPlaceV2 &networkPlace);

    int m_id;
    QString m_name;
    QString m_url;
    QString m_localPath;
    Flags m_flags;
};

}

Q_DECLARE_METATYPE(FlyFM::NetworkPlaceV2)
Q_DECLARE_METATYPE(FlyFM::NetworkPlaceListV2)
Q_DECLARE_METATYPE(FlyFM::NetworkPlaceV2::Flag)
Q_DECLARE_METATYPE(FlyFM::NetworkPlaceV2::Flags)
Q_DECLARE_OPERATORS_FOR_FLAGS(FlyFM::NetworkPlaceV2::Flags)

#endif // FLYFM_NETWORKPLACEV2_H
