/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMPATHANALYZER_H
#define FLYFMPATHANALYZER_H

#include <flyfm/core_export.h>

#include <QString>
#include <QQueue>

#include <flyfm/entry.h>
#include <flyfm/entrylist.h>
#include <flyfm/pathanalyzerpair.h>

namespace FlyFM
{

class FLYFM_CORE_EXPORT PathAnalyzer
{
public:
    enum Error { NoError = 0, DirToFileError = 1, EntriesToFileError = 2 };
    enum Mode { DefaultMode = 0, DirectMode = 1, CopyMode = 2 };

    explicit PathAnalyzer(const EntryList &sources = EntryList(), const Entry &destination = Entry(), Mode mode = DefaultMode);
    ~PathAnalyzer();

    void resolve(const EntryList &sources, const Entry &destination = Entry(), Mode mode = DefaultMode);
    void stopResolving();

    void clear();

    EntryList sources() const;

    Error error() const;
    QString errorText() const;

    bool isSourceEntry(const Entry &entry) const;

    void prepend(const PathAnalyzerPair &pair);
    PathAnalyzerPair getNext();
    void removeNext();
    void removeChildren(const Entry &parentDir);
    quint64 totalSize() const;
    int count(bool fileOnly = false) const;
    bool isEmpty() const;

    static bool isValid(const EntryList &sources);

protected:
    void resolveEntry(const Entry &source, const Entry &destination, Mode mode);
    bool hasToStopResolving();

    EntryList m_sources;
    QQueue<PathAnalyzerPair> m_sequence;
    quint64 m_totalSize;

    Error m_error;
    Mode m_mode;
    QString m_errorText;

    QAtomicInt m_dontDoResolve;
    QAtomicInt m_stopGettingChildsFromVFS;

    friend class PathAnalyzerPair;
};

}

#endif // FLYFMPATHANALYZER_H
