#ifndef FLYFM_PATHANALYZERPAIR_H
#define FLYFM_PATHANALYZERPAIR_H

#include <flyfm/core_export.h>

#include <QString>

#include <flyfm/entry.h>

namespace FlyFM
{

class FLYFM_CORE_EXPORT PathAnalyzerPair
{
    friend class PathAnalyzer;
public:
    enum Mode { DefaultMode = 0, DirectMode = 1 };

    PathAnalyzerPair();
    PathAnalyzerPair &operator=(const PathAnalyzerPair &);
    ~PathAnalyzerPair();

    Entry source() const;
    Entry destination() const;

    QString sourcePath() const;
    QString sourcePrefix() const;
    QString sourceSuffix(bool withSlash = false) const;
    QString sourceLocalPath() const;

    QString destinationPath() const;
    QString destinationPrefix() const;
    QString destinationSuffix(bool withSlash = false) const;
    QString destinationLocalPath() const;

    bool isFiles() const;

protected:
    PathAnalyzerPair(const QString &sourcePrefix, const Entry &source, const Entry &destination, Mode mode = DefaultMode);

    QString sourceTailDirPath(bool withSlash = false) const;
    QString sourceTailFileName(bool withSlash = false) const;
    QString destinationTailDirPath(bool withSlash = false) const;
    QString destinationTailFileName(bool withSlash = false) const;

    static QString cutFileName(const QString &path);
    static QString replaceTokenFirstByLast(const QString &from, const QString &to);
    static QString slashed(const QString path, bool withSlash);
    static bool isDirKindEntry(const Entry &entry);
    static bool isFileKindEntry(const Entry &entry);

    void tryResolve() const;
    void resolve() const;

    QString m_initialSourcePrefix;

    Entry m_source;
    Entry m_destination;

    Mode m_mode;

    // to be resolved
    mutable QString m_sourcePrefix;
    mutable QString m_sourceTailDirPath;
    mutable QString m_sourceTailFileName;

    mutable QString m_destinationPath;
    mutable QString m_destinationPrefix;
    mutable QString m_destinationTailDirPath;
    mutable QString m_destinationTailFileName;

    mutable bool m_isResolved;
};

} // namespace FlyFM

#endif // FLYFM_PATHANALYZERPAIR_H
