/*

  Graphic user's interface library
  Places widget

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFM_PLACESWIDGET_H
#define FLYFM_PLACESWIDGET_H

#include <flyfm/gui_export.h>
#include <flyfm/treeview.h>
#include <flyfm/imanageiconsize.h>

class QListWidgetItem;

namespace FlyFM
{

class Entry;
class EntryList;
class Navigation;
class PlacesModel;
class IconSizeActionsProvider;
class ItemViewTouchGesturesManager;

class FLYFM_GUI_EXPORT PlacesWidget : public ScheduledUpdateTreeView, public IManageIconSize
{
    Q_OBJECT
public:
    PlacesWidget(QWidget *parent = nullptr);
    ~PlacesWidget();

    void setNavigation(Navigation *navigation);

    IconSize iconSize() const override;

    QSize sizeHint() const override;

    static QByteArray defaultState(bool tabletMode);
    QByteArray saveState() const;
    void restoreState(const QByteArray &state);

    const Entry rootEntry() const;

    PlacesModel *model();
    const PlacesModel *model() const;

    void expandFavourites();

public slots:
    void setIconSize(IconSize size) override;
    void setCurrentEntry(const Entry &entry);

signals:
    void entrySelected(const Entry &);

protected slots:
    void dropPopupContextMenu(const QPoint &, Qt::DropActions, const EntryList &, const Entry &);
    void arkExtractContextMenu(const QPoint &, const QString &dbusService, const QString &dbusPath, const Entry &dst);
    void dropTitledShortcutMenu(const QPoint &pos, const QString &url, const QString &title, const Entry &dst);
    void customPopupContextMenu(const QPoint &, const QModelIndexList &);
    void checkNewSelection(const QItemSelection &selection);
    void activatedAction(const QModelIndex &);

    void increaseIconSizeAction();
    void decreaseIconSizeAction();

protected:
    bool viewportEvent(QEvent *event) override;

    void openAction(const QModelIndex &index);

    QMap<QListWidgetItem *, Entry> m_itemEntryMap;

    PlacesModel *m_model;
    IconSize m_iconSize;
    Navigation *m_navigation;

    IconSizeActionsProvider *m_iconSizeActionsProvider;

    ItemViewTouchGesturesManager *m_touchGesturesManager;
};

}

#endif // FLYFM_PLACESWIDGET_H
