/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMPOPUPMENU_H
#define FLYFMPOPUPMENU_H

#include <QMenu>

#include <flyfm/entrylist.h>
#include <flyfm/actionplugin.h>
#include <flyfm/storageplace.h>
#include <flyfm/fops.h>
#include <flyfm/favouriteplace.h>

class QAction;

namespace FlyFM
{

class DesktopWidget;
class TreeWidget;
class FolderWidget;
class Navigation;

class ViewMenu;
class SortMenu;
class ColumnMenu;

class IconSizeActionsProvider;
class FileActionsProvider;
class ViewsTabWidget;

class PopupMenu : public QMenu
{
    Q_OBJECT
public:
    enum EntryInterpretation { Folder = 0, Contents = 1 };

    enum Option {
        HideOpenActions = 0x00000001,
        HideFindAction  = 0x00000002,
        HideSendActions = 0x00000004,
        HideFileActions = 0x00000008,
        HideViewChooseActions = 0x00000010,
        HideShowHiddenAction = 0x00000020
    };
    Q_DECLARE_FLAGS(Options, Option)

    PopupMenu(QWidget *parent = nullptr);
    ~PopupMenu();

    void setConditions(const EntryList &entries, const Entry &parentEntry, EntryInterpretation entryInterpretation);
    void setOptions(Options options);

    void setFolderWidget(FolderWidget *folderWidget);
    void setDesktopWidget(DesktopWidget *desktopWidget);
    void setTreeWidget(TreeWidget *treeWidget);
    void setNavigation(Navigation *pNavigation);

    void create();

    static ViewMenu *createViewChooseMenu(FolderWidget *pFolderWidget, QWidget *parent = nullptr);
    static SortMenu *createSortChooseMenu(FolderWidget *pFolderWidget, QWidget *parent = nullptr);
    static ColumnMenu *createColumnChooseMenu(FolderWidget *pFolderWidget, QWidget *parent = nullptr);

    void setIconSizeActionsProvider(IconSizeActionsProvider *iconSizeActionsProvider);
    void setFileActionsProvider(FileActionsProvider *fileActionsProvider);

signals:
    void possibleEntryPosition(const EntryList &, const QPoint &);

protected slots:
    void makeOpenAction();
    void makeOpenInNewTabAction();
    void makeOpenInOtherPanel();
    void makeOpenWithAction();
    void makeOpenContainingFolderAction();
    // void makeShortcutsAction();

    //void makeTrashClipboardAction();
    void makePropsAction();
    // void makeSendDesktopAction();
    void makeSendDesktopShortcutsAction();
    void makeSendHomeDirAction();
    void makeSendDocumentsAction();
    void makeSendStartMenuAction();
    void makeSendStartPanelAction();
    void makeSendAutostartAction();
    void makeSendFavouritesAction();
    void makeSendBrowseAction();
    // void makeSendFloppyAction();
    void makeSendMailAction();
    void makeMountAction();
    void makeUnmountAction();
    void makeCommitArchiveAction();
    void makeEjectAction();
    void makeFindInAction();
    void makeFindAction();
    void makeFollowLinkAction();
    void makeNewNetworkPlace(const QString &url = QString());
    void makeNewNetworkPlaceFromCurrent();
    void makeRemoveNetworkPlace();

    void snapToGridAction(bool);
    void arrangeItemsAction();

    void highlightEntry(const Entry &);
    void goToEntry(const Entry &);

    void updateMountActions(const StoragePlace &);

    void addCurrentLocationToFavourites();
    void removeCurrentLocationFromFavourites();

    void fillActionsSubmenu();

protected:
    bool isInFindMode() const;
    void makeSendStorageAction(const StoragePlace &place);

    DesktopWidget *m_desktopWidget;
    FolderWidget *m_folderWidget;
    TreeWidget *m_treeWidget;
    Navigation *m_pNavigation;

    PluginActionPtrList m_actionSubmenuActions;
    PluginActionPtrList m_sendSubmenuActions;

    QMenu *m_actionsSubmenu;
    bool m_actionsSubmenuFilled;

    EntryInterpretation m_entryInterpretation;
    EntryList m_entries;
    Entry m_parentEntry;

    StoragePlace m_storagePlace;
    FavouritePlace m_favouritePlace;

    QAction *m_mountAction;
    QAction *m_unmountAction;

    IconSizeActionsProvider *m_iconSizeActionsProvider;
    FileActionsProvider *m_fileActionsProvider;

    Options m_options;

private:
    void addFindAction();
    void addCreateSubmenu();

    void addFileActions();

    void addSendSubmenu();
    void createActionsSubmenu();

    void addNewNetworkPlaceAction();
    void addNewNetworkPlaceFromCurrentAction();
    void addRemoveNetworkPlaceAction();

    void addCommitArchiveAction();
    void addShowHiddenAction();

    void addPropertiesAction();
    void addConfigureTreeAction();

    bool canOpenAsDir() const;
    ViewsTabWidget *viewsTabWidget() const;
};

}

Q_DECLARE_OPERATORS_FOR_FLAGS(FlyFM::PopupMenu::Options)

#endif // FLYFMPOPUPMENU
