/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMPROPERTYDATAHOLDER_H
#define FLYFMPROPERTYDATAHOLDER_H

#include <flyfm/core_export.h>

#include <QHash>
#include <QVariant>
#include <QReadWriteLock>
#include <QVector>

#include <atomic>

namespace FlyFM
{

class Entry;
class PropertyEnumerator;
class EntryPropertySet;

class FLYFM_CORE_EXPORT PropertyDataHolder
{
public:
    QVariant propertyValueData(const Entry &entry, int index, int role, const PropertyEnumerator *propertyEnumerator);

    void invalidate(int sortIndex, const PropertyEnumerator *propertyEnumerator);
    void updatePresentation(const Entry &entry);
    void delayedRefineAllProperties(const Entry &entry, PropertyEnumerator *propertyEnumerator);
    void refineAllProperties(const Entry &entry, PropertyEnumerator *propertyEnumerator);

    void refinePropertyAtIndex(int index, const PropertyEnumerator *propertyEnumerator);

    bool isRefined() const;

protected:
    PropertyDataHolder(int nPropertySets);
    ~PropertyDataHolder();

    void incrementReferenceCount();
    bool decrementReferenceCount();

    void setRefined();

    EntryPropertySet *preparePropertySet(const Entry &entry, int setIndex, const PropertyEnumerator *propertyEnumerator);

    QVector<EntryPropertySet *> m_propertySets;
    using PropertyFlags = uint_fast32_t;
    std::atomic<PropertyFlags> m_isPropertySetInited;
    std::atomic<PropertyFlags> m_isPropertySetRefined;

    void setBit(std::atomic<PropertyFlags> *bits, int i);
    void clearBit(std::atomic<PropertyFlags> *bits, int i);
    bool testBit(std::atomic<PropertyFlags> *bits, int i);

    QAtomicInt m_refCount;

    friend class PropertyEnumerator;
};

using PropertyDataHolderPtrList = QList<PropertyDataHolder *>;
}

#endif // FLYFMROPERTYDATAHOLDER_H
