/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMPROPERTYENUMERATOR_H
#define FLYFMPROPERTYENUMERATOR_H

#include <flyfm/core_export.h>

#include <QMap>
#include <QPointer>
#include <QVector>
#include <QLinkedList>
#include <QElapsedTimer>

#include <flyfm/thread.h>
#include <flyfm/entry.h>

class QTimer;

template <typename T>
class QFutureWatcher;
template <>
class QFutureWatcher<void>;

namespace FlyFM
{

class PropertyDataHolder;
class EntryPropertySet;
class PropsPlugin;

//////////////////////////////////////////////////////////////////

class EntryPropertyRequest
{
public:
    EntryPropertyRequest();
    EntryPropertyRequest(const QVector<EntryPropertySet *> &propertySets, PropertyDataHolder *dataHolder, const Entry &entry);
    ~EntryPropertyRequest();

    QVector<EntryPropertySet *> propertySets() const;
    PropertyDataHolder *dataHolder() const;
    Entry entry() const;

protected:
    QVector<EntryPropertySet *> m_propertySets;
    PropertyDataHolder *m_dataHolder;
    Entry m_entry;
};

//////////////////////////////////////////////////////////////////

class EntryPropertyRequestList : public QList<EntryPropertyRequest>
{
public:
    EntryPropertyRequestList();
    explicit EntryPropertyRequestList(const EntryPropertyRequest &entryPropertyRequest);
    ~EntryPropertyRequestList();

    int removeAllRequestsFor(const PropertyDataHolder *dataHolder);
};

//////////////////////////////////////////////////////////////////

class FLYFM_CORE_EXPORT PropertyEnumerator : public QObject
{
    Q_OBJECT
public:
    explicit PropertyEnumerator(const QList<PropsPlugin *> &propsPlugins, QObject *parent = nullptr);
    virtual ~PropertyEnumerator();

    PropertyDataHolder *allocPropertyDataHolder(const Entry &/*, int iconSize = 32*/);
    void releasePropertyDataHolder(PropertyDataHolder *);

    QVariant propertyNameData(int index, int role);

    bool isSortableProperty(int);
    int count() const;

    void startProcessing();
    void stopProcessing();

    QStringList emblems(const Entry &entry, const QVector<EntryPropertySet *> &propertySets) const;

    void setShowLinkEmblem(bool on = true);
    bool showLinkEmblem() const;

    void setShowPropertyEmblems(bool on = true);
    bool showPropertyEmblems() const;

    void updatePropertySets(const QVector<EntryPropertySet *> &propertySets, PropertyDataHolder *dataHolder, const Entry &entry);

    QString displayedName(const Entry &entry);
    QIcon icon(const Entry &entry);

signals:
    void dataHoldersUpdated(const EntryPropertyRequestList &);

protected slots:
    void releaseDataHoldersTimeout();
    void flushUpdatedHolders();
    void holderUpdated();

protected:
    void processNextHolder();

    void releaseDataHolders(int remainingLimit);

    void removeSelfRequests(const PropertyDataHolder *pPropertyDataHolder);
    void removeSelfUpdates(const PropertyDataHolder *pPropertyDataHolder);

    static void removeSelfRequestsForAllEnumerators(const PropertyDataHolder *pPropertyDataHolder);
    static void removeSelfUpdatesForAllEnumerators(const PropertyDataHolder *pPropertyDataHolder);

    static EntryPropertyRequest updateHolder(const EntryPropertyRequest &request);

    int indexInPropertySet(int propertyIndex, int *propertySetIndex) const;
    QVector<int> propertySetsProvidingEmblems() const;
    EntryPropertySet *createPropertySet(int propertySetIndex) const;

    QList<PropsPlugin *> m_propsPlugins;

    EntryPropertyRequestList m_requestList;
    EntryPropertyRequestList m_updatedHoldersRequestList;

    QFutureWatcher<EntryPropertyRequest> *m_updateHolderWatcher;

    QElapsedTimer m_flushUpdatesTimer;

    QMap<QString, PropertyDataHolder *> m_entryDataHolderPtrMap;
    QLinkedList<PropertyDataHolder *> m_unreferencedDataHolders;
    QMap<PropertyDataHolder *, QLinkedList<PropertyDataHolder *>::iterator> m_unreferencedDataHoldersMap;

    QTimer *m_releaseDataHoldersTimer;

    bool m_isUpdatingHolder;
    int m_isProcessing;

    bool m_showLinkEmblem;
    bool m_showPropertyEmblems;

    static QList<QPointer<PropertyEnumerator>> m_enumerators;
    static void cleanOutdatedEnumerators();

    friend class Props;
    friend class PropertyDataHolder;
};
}

#endif // FLYFMROPERTYENUMERATOR_H
