/*

  File system entry properties library
  Application programmer's interface

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFM_PROPS_H
#define FLYFM_PROPS_H

#include <flyfm/core_export.h>

#include <QScopedPointer>

#include <flyfm/singletone.h>

class QIcon;
template <class T> class QList;
template <class Key, class T> class QMap;

namespace FlyFM
{

class Entry;
class TextDigestEnumerator;
class PropertyEnumerator;

class PropsPrivate;

class FLYFM_CORE_EXPORT Props : public Singletone
{
public:
    enum AccessMode { ReadMode, WriteMode, ExecMode };

    static Props *ptr();
    virtual ~Props();

    void release() override;

    QString displayedName(const Entry &entry) const;
    QIcon icon(const Entry &entry) const;

    PropertyEnumerator *propertyEnumerator();

    TextDigestEnumerator *createTextDigestEnumerator();
    void releaseTextDigestEnumerator(TextDigestEnumerator *);

private:
    Props();
    Props(const Props &);
    Props &operator= (const Props &);
    static Props *m_ptr;

    QScopedPointer<PropsPrivate> d;
};
}

#endif // FLYFM_PROPS_H
