/*

  File system entry properties library
  Plugin abstract class

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMPROPSPLUGIN_H
#define FLYFMPROPSPLUGIN_H

#include <QtPlugin>

#include <flyfm/core_export.h>
#include <flyfm/plugin.h>
#include <flyfm/entrylist.h>

namespace FlyFM
{

class PropsPlugin;
class PropsPluginWidget;
class FindPluginWidget;
class FilterPluginWidget;
class EntryPropertySet;

using PropsPluginPtrList = QList<PropsPlugin *>;

class FLYFM_CORE_EXPORT PropsPlugin : public Plugin
{
public:
    PropsPlugin();
    virtual ~PropsPlugin();

    virtual QString label() = 0;
    virtual int orderPriority();

    virtual bool hasTextDigest();
    virtual QString textDigest(const EntryList &entries, bool *m_pAliveFlag = nullptr);

    virtual bool hasFindWidget();
    virtual bool hasPropsWidgetFor(const EntryList &);

    virtual QList<FindPluginWidget *> createFindWidgets(QWidget *parent = nullptr);
    virtual QList<FilterPluginWidget *> createFilterWidgets(QWidget *parent = nullptr);
    virtual PropsPluginWidget *createPropsWidget(const EntryList &entries, QWidget *parent = nullptr);

    virtual int propertySetSize() const;
    virtual QVariant propertyHeaderData(int index, int role);
    virtual bool isPropertySortable(int index);
    virtual EntryPropertySet *createPropertySet() const;
    virtual bool providesEmblems() const;
};

}

#define FlyFM_PropsPlugin_iid FLY_ORG_DOMAIN".fly-fm.PropsPlugin/1.0"

Q_DECLARE_INTERFACE(FlyFM::PropsPlugin, FlyFM_PropsPlugin_iid)

#endif // FLYFMPROPSPLUGIN_H
