/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMSERVICE_H
#define FLYFMSERVICE_H

#include <flyfm/core_export.h>

#include <QObject>
#include <QDBusContext>
#include <QTimer>

#include <flyfm/core.h>

class QDBusServiceWatcher;

namespace FlyFM
{

void haltService(Core::HaltReason reason);

class FLYFM_CORE_EXPORT Service : public QObject, protected QDBusContext
{
    Q_OBJECT
public:
    // enum UnloadMode { UnloadByCall = 0, UnloadByProcess = 1 };

    Service(QObject *parent);
    virtual ~Service();

    virtual void haltHandler(Core::HaltReason reason);

    void lockUnloader();
    void recallUnloader();
    void unlockUnloader();

public Q_SLOTS:
    void capture();

protected slots:
    virtual void release() = 0;
    void unloaderTimeout();
    void slotCallingServiceUnregistered(const QString &service);

protected:
    virtual bool unloadIsAllowed();
    void wrappedHaltHandler(Core::HaltReason reason);

    QTimer m_unloaderTimer;
    int m_unloaderRefsCount;

    QDBusServiceWatcher *m_callingServicesWatcher;

    friend void haltService(Core::HaltReason);
};

class ServiceLocker
{
    Service *m_service;
public:
    ServiceLocker(Service *service) : m_service(service)
    {
        m_service->lockUnloader();
    }

    ~ServiceLocker()
    {
        m_service->unlockUnloader();
    }
};

}

#endif // FLYFMSERVICE_H
