#ifndef FLYFM_SMBBROWSERPROXY_H
#define FLYFM_SMBBROWSERPROXY_H

#include <QObject>
#include <QSharedPointer>
#include <QStringList>
#include <QMap>

#include <flyfm/stringpair.h>

class QDBusInterface;

namespace FlyFM
{

class SmbBrowserProxy : public QObject
{
    Q_OBJECT
public:
    explicit SmbBrowserProxy(QWeakPointer<QDBusInterface> interface, QObject *parent = nullptr);

public slots:
    void rescan();
    void rescanWorkgroup(const QString &workgroup);
    void rescanHost(const QString &host, const QString &workgroup, const QString &user);

    QStringList workgroups() const;
    QStringList hosts(const QString &workgroup) const;
    StringMap shares(const QString &host, const QString &workgroup) const;

    QString smbNetworkLocalPath() const;
    QString shareLocalPath(const QString &share, const QString &host, const QString &workgroup, const QString &hostForKerberos) const;

signals:
    void workgroupsChanged(const QStringList &addedWorkgroups, const QStringList &removedWorkgroups);
    void workgroupsNotChanged();
    void workgroupUpdated(const QString &workgroup, const QStringList &addedHosts, const QStringList &removedHosts);
    void workgroupNotUpdated(const QString &workgroup);
    void hostUpdated(const QString &host, const QString &workgroup, const StringMap &addedShares, const StringMap &removedShares);
    void hostNotUpdated(const QString &host, const QString &workgroup);

    void shareUpdated(const QString &share, const QString &host, const QString &workgroup, const QString &localPath);

    void shareAccessDenied(const QString &share, const QString &host, const QString &workgroup);
    void shareLogonFailure(const QString &share, const QString &host, const QString &workgroup);
    void shareAccessGranted(const QString &share, const QString &host, const QString &workgroup);

private slots:
    void slotRescanFinished(bool changed);
    void slotRescanWorkgroupFinished(const QString &workgroup, bool changed);
    void slotRescanHostFinished(const QString &host, const QString &workgroup, bool changed);
    void slotSharedUpdated(const QString &share, const QString &host, const QString &workgroup, const QString &localPath);

private:
    void processShareCheck();

private:
    QWeakPointer<QDBusInterface> m_interface;

    struct Shares {
        Shares() : cached(false) {}

        StringMap shares;
        bool cached;
    };

    struct Hosts {
        Hosts() : cached(false) {}

        QMap<QString, Shares> hosts;
        bool cached;
    };

    using WorkgroupsMap = QMap<QString, Hosts>;

    mutable WorkgroupsMap m_workgroupsMap;
    mutable bool m_workgroupsCached;
};

} // namespace FlyFM

#endif // FLYFM_SMBBROWSERPROXY_H
