#ifndef STORAGEDEVICEINFO_H
#define STORAGEDEVICEINFO_H

#include <QMetaType>
#include <QString>

class QDBusArgument;

namespace FlyFM
{
class StorageDeviceInfo;
}

QDBusArgument &operator<<(QDBusArgument &arg, const FlyFM::StorageDeviceInfo &info);
const QDBusArgument &operator>>(const QDBusArgument &arg, FlyFM::StorageDeviceInfo &info);

namespace FlyFM
{

class StorageDeviceInfo
{
    friend QDBusArgument &::operator<<(QDBusArgument &arg, const StorageDeviceInfo &info);
    friend const QDBusArgument &::operator>>(const QDBusArgument &arg, StorageDeviceInfo &info);
public:
    StorageDeviceInfo()
        : m_isVolume(false), m_volumeSize(0), m_volumeFreeSize(0),
          m_hasQuota(false), m_hardQuota(0), m_softQuota(0), m_hasCurrentlyUsedQuota(false), m_currentlyUsedQuota(0)
    {}

    QString model() const
    {
        return m_model;
    }
    void setModel(const QString &model)
    {
        m_model = model;
    }

    QString serial() const
    {
        return m_serial;
    }
    void setSerial(const QString &serial)
    {
        m_serial = serial;
    }

    QString bus() const
    {
        return m_bus;
    }
    void setBus(const QString &bus)
    {
        m_bus = bus;
    }

    QString blockDevice() const
    {
        return m_blockDevice;
    }
    void setBlockDevice(const QString &blockDevice)
    {
        m_blockDevice = blockDevice;
    }

    bool isVolume() const
    {
        return m_isVolume;
    }
    void setIsVolume(bool isVolume)
    {
        m_isVolume = isVolume;
    }

    QString volumeMountPoint() const
    {
        return m_volumeMountPoint;
    }
    void setVolumeMountPoint(const QString &volumeMountPoint)
    {
        m_volumeMountPoint = volumeMountPoint;
    }

    QString volumeFSType() const
    {
        return m_volumeFSType;
    }
    void setVolumeFSType(const QString &volumeFSType)
    {
        m_volumeFSType = volumeFSType;
    }

    QString volumeFSVersion() const
    {
        return m_volumeFSVersion;
    }
    void setVolumeFSVersion(const QString &volumeFSVersion)
    {
        m_volumeFSVersion = volumeFSVersion;
    }

    QString volumeBlockSize() const
    {
        return m_volumeBlockSize;
    }
    void setVolumeBlockSize(const QString &volumeBlockSize)
    {
        m_volumeBlockSize = volumeBlockSize;
    }

    QString volumeLabel() const
    {
        return m_volumeLabel;
    }
    void setVolumeLabel(const QString &volumeLabel)
    {
        m_volumeLabel = volumeLabel;
    }

    QString volumeUUID() const
    {
        return m_volumeUUID;
    }
    void setVolumeUUID(const QString &volumeUUID)
    {
        m_volumeUUID = volumeUUID;
    }

    quint64 volumeSize() const
    {
        return m_volumeSize;
    }
    void setVolumeSize(quint64 volumeSize)
    {
        m_volumeSize = volumeSize;
    }

    quint64 volumeFreeSize() const
    {
        return m_volumeFreeSize;
    }
    void setVolumeFreeSize(quint64 volumeFreeSize)
    {
        m_volumeFreeSize = volumeFreeSize;
    }

    bool hasQuota() const
    {
        return m_hasQuota;
    }
    void setHasQuota(bool hasQuota)
    {
        m_hasQuota = hasQuota;
    }

    quint64 hardQuota() const
    {
        return m_hardQuota;
    }
    void setHardQuota(quint64 hardQuota)
    {
        m_hardQuota = hardQuota;
    }

    quint64 softQuota() const
    {
        return m_softQuota;
    }
    void setSoftQuota(quint64 softQuota)
    {
        m_softQuota = softQuota;
    }

    bool hasCurrentlyUsedQuota() const
    {
        return m_hasCurrentlyUsedQuota;
    }
    void setHasCurrentlyUsedQuota(bool hasCurrentlyUsedQuota)
    {
        m_hasCurrentlyUsedQuota = hasCurrentlyUsedQuota;
    }

    quint64 currentlyUsedQuota() const
    {
        return m_currentlyUsedQuota;
    }
    void setCurrentlyUsedQuota(quint64 currentlyUsedQuota)
    {
        m_currentlyUsedQuota = currentlyUsedQuota;
    }

private:
    QString m_model;
    QString m_serial;
    QString m_bus;
    QString m_blockDevice;
    bool m_isVolume;
    QString m_volumeMountPoint;
    QString m_volumeFSType;
    QString m_volumeFSVersion;
    QString m_volumeBlockSize;
    QString m_volumeLabel;
    QString m_volumeUUID;
    quint64 m_volumeSize;
    quint64 m_volumeFreeSize;
    bool m_hasQuota;
    quint64 m_hardQuota;
    quint64 m_softQuota;
    bool m_hasCurrentlyUsedQuota;
    quint64 m_currentlyUsedQuota;
};

}

Q_DECLARE_METATYPE(FlyFM::StorageDeviceInfo)

#endif
