#ifndef STORAGEPLACESMANAGERPROXY_H
#define STORAGEPLACESMANAGERPROXY_H

#include <QObject>
#include <QSharedPointer>

#include <flyfm/storagedeviceinfo.h>
#include <flyfm/storageplace.h>

class QDBusInterface;

namespace FlyFM
{

class StoragePlaceMountJob;
class StoragePlaceUnmountJob;
class StoragePlaceEjectJob;

class StoragePlacesManagerProxy : public QObject
{
    Q_OBJECT
public:
    explicit StoragePlacesManagerProxy(QWeakPointer<QDBusInterface> interface, QObject *parent = nullptr);

    StoragePlaceList availableStoragePlaces();

    StoragePlaceMountJob *mountStoragePlace(const StoragePlace &storagePlace);
    StoragePlaceUnmountJob *unmountStoragePlace(const StoragePlace &storagePlace);
    StoragePlaceEjectJob *ejectStoragePlaceDevice(const StoragePlace &storagePlace);

    StorageDeviceInfo storageDeviceInfo(const StoragePlace &storagePlace) const;

signals:
    void mountError(const QString &error);
    void unmountError(const QString &error);
    void ejectError(const QString &error);

    void storagePlaceUpdated(const FlyFM::StoragePlace &, const FlyFM::StoragePlace &);
    void storagePlaceAdded(const FlyFM::StoragePlace &);
    void storagePlaceRemoved(const FlyFM::StoragePlace &);

private slots:
    void slotStoragePlaceUpdated(const FlyFM::StoragePlace &place);
    void slotStoragePlaceAdded(const FlyFM::StoragePlace &place);
    void slotStoragePlaceRemoved(const FlyFM::StoragePlace &place);

private:
    QWeakPointer<QDBusInterface> m_interface;

    StoragePlaceList m_places;
    bool m_cached;
};

}

#endif // STORAGEPLACESMANAGERPROXY_H
