/*

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMTRASH_H
#define FLYFMTRASH_H

#include <flyfm/core_export.h>

#include <QString>
#include <QDateTime>

namespace FlyFM
{
class Entry;
class EntryList;

class FLYFM_CORE_EXPORT Trash
{
public:
    static Trash *ptr();

    bool registerFile(const QString &filePath, QString &destPath);
    bool unregisterFile(const QString &filePath);

    QString registeredFilePath(const QString &trashFilePath) const;
    QDateTime deletionDate(const QString &trashFilePath) const;
    QString actualIconName() const;

    bool hasEntries(const QStringList &urls) const;
    bool isTrash(const QString &url) const;

    bool hasEntries(const EntryList &entries) const;
    bool isTrash(const Entry &entry) const;
    bool isEmpty() const;

    qint64 size() const;
    void registerDirSize(const QString &dirPath, qint64 size = -1);
    void unregisterDirSize(const QString &dirPath);
    bool hasSizeCache() const;

protected:
    Trash();
    Trash(const Trash &) = delete;
    Trash &operator=(const Trash &) = delete;

    static Trash *m_ptr;

private:
    void registerDirSizeHelper(const QString &dirPath, qint64 size, bool reg);
};
}

#endif // FLYFMTRASH_H
