#ifndef FLYFM_TREEVIEWMOD_H
#define FLYFM_TREEVIEWMOD_H

#include <QTreeView>

namespace FlyFM
{

class TreeViewModPrivate;

class TreeViewMod : public QTreeView
{
    Q_OBJECT
public:
    explicit TreeViewMod(QWidget *parent = nullptr);
    ~TreeViewMod();

    QRect visualRect(const QModelIndex &index) const override;

    void setHighlightedIndex(const QModelIndex &index);
    QModelIndex highlightedIndex() const;

    void setShowSelection(bool on);
    bool showSelection() const;

protected:
    void drawRow(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override;
    void drawBranches(QPainter *painter, const QRect &rect, const QModelIndex &index) const override;
    int sizeHintForColumn(int column) const override;

    bool viewportEvent(QEvent *event) override;
    void mousePressEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
    void mouseDoubleClickEvent(QMouseEvent *event) override;
    void mouseMoveEvent(QMouseEvent *event) override;

private:
    Q_DECLARE_PRIVATE(QTreeView)

    bool isInNoOrEditingState() const;

    friend class TreeViewModPrivate;
    QScopedPointer<TreeViewModPrivate> p;
};

}

#endif
