/*

  Graphic user's interface library
  Tree widget

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFMTREEWIDGET_H
#define FLYFMTREEWIDGET_H

#include <flyfm/gui_export.h>

#include <QWidget>
#include <QModelIndexList>
#include <QVector>
#include <QPointer>

#include <flyfm/entrylist.h>
#include <flyfm/imanageiconsize.h>

namespace FlyFM
{
class Navigation;
class ScheduledUpdateTreeView;
class Model;
class PathCompleter;
class IconSizeActionsProvider;
class FileActionsProvider;
class Job;
class ItemViewTouchGesturesManager;

class FLYFM_GUI_EXPORT TreeWidget : public QWidget, public IManageIconSize
{
    Q_OBJECT
public:
    explicit TreeWidget(Navigation *navigation, QWidget *parent = nullptr);
    ~TreeWidget();

    void setRootEntry(const Entry &entry);
    const Entry rootEntry() const;

    Model *model();
    const Model *model() const;

    Entry currentEntry() const;

    PathCompleter *createPathCompleter();

    void setHeaderHidden(bool on);

    IconSize iconSize() const override;

    static QByteArray defaultState(bool tabletMode);
    QByteArray saveState() const;
    void restoreState(const QByteArray &state);

    bool showHiddenEntries() const;
    void setShowHiddenEntries(bool enable);

    void setNavigation(Navigation *navigation);

    EntryList selectedEntries() const;

    void setHideShowHiddenAction(bool enable);

    void setHiddenPlaces(const EntryList &entries);

    FileActionsProvider *fileActionsProvider();

signals:
    void entriesSelected(const EntryList &);

public slots:
    void setCurrentEntry(const Entry &entry);
    void highlightEntry(const Entry &entry);
    void setIconSize(IconSize size) override;

protected slots:
    void dropPopupContextMenu(const QPoint &, Qt::DropActions, const EntryList &, const Entry &);
    void arkExtractContextMenu(const QPoint &, const QString &dbusService, const QString &dbusPath, const Entry &dst);
    void dropTitledShortcutMenu(const QPoint &pos, const QString &url, const QString &title, const Entry &dst);
    void customPopupContextMenu(const QPoint &, const QModelIndexList &);
    void openAction(const QModelIndexList &indexes);
    void upAction();

    void increaseIconSizeAction();
    void decreaseIconSizeAction();
    void selectionChanged();

    void modelReset();
    void updateModelPresentation();

    void updateShortcutFileAction();

    void doubleClickedSlot(const QModelIndex &index);
    void clickedSlot(const QModelIndex &index);
    void enterPressedSlot();

protected:
    bool eventFilter(QObject *watched, QEvent *event) override;

protected:
    Navigation *m_navigation;
    ScheduledUpdateTreeView *m_view;
    Model *m_model;
    IconSize m_iconSize;

    IconSizeActionsProvider *m_iconSizeActionsProvider;
    FileActionsProvider *m_fileActionsProvider;

    ItemViewTouchGesturesManager *m_touchGesturesManager;

    bool m_hideShowHiddenAction;

    EntryList m_hiddenPlaces;

    struct NavigationScrollPos
    {
        QPointer<Navigation> navigation;
        QPoint scrollPos;
    };
    QVector<NavigationScrollPos> m_scrollPositions;
};
}

#endif // FLYFMTREEWIDGET_H
