#ifndef FLYFM_VFSSETTINGS_H
#define FLYFM_VFSSETTINGS_H

#include <flyfm/vfs_export.h>
#include <flyfm/networkplacev2.h>

#include <QObject>
#include <QString>
#include <QScopedPointer>

class KConfig;
class QFileSystemWatcher;

namespace FlyFM
{

class VFSPrivate;

class FLYFM_VFS_EXPORT VFSSettings : public QObject
{
    Q_OBJECT
public:
    VFSSettings();
    ~VFSSettings();

    void sync();

    QString mountRootPath() const;
    bool isNetworkEnabled() const;
    bool isNetworkSlow() const;
    bool isNetworkExternalMountsSlow() const;
    bool useSmbtree() const;
    QString defaultSmbUser() const;

    struct ArchiveExtensionInfo
    {
        QString extension;
        bool readOnly;
    };
    using ArchiveExtensions = QList<ArchiveExtensionInfo>;
    ArchiveExtensions archiveExtensions() const;
    void setArchiveExtensions(const ArchiveExtensions &extensions);
    bool isArchivesEnabled() const;

    bool autoDetectBadPaths() const;
    void setAutoDetectBadPaths(bool on);

    QStringList badPaths() const;
    void addBadPath(const QString &path);
    void setBadPaths(const QStringList &paths);

    NetworkPlaceListV2 networkPlaces() const;
    void setNetworkPlaces(const NetworkPlaceListV2 &places);

private slots:
    void refresh();

private:
    void refreshSettingsWatcher();

    QScopedPointer<KConfig> m_settings;
    QFileSystemWatcher *m_settingsWatcher;
    ArchiveExtensions m_archiveExtensions;
    bool m_isArchivesEnabled;
    bool m_isNetworkEnabled;
    bool m_isNetworkSlow;
    bool m_isNetworkExternalMountsSlow;
    bool m_useSmbtree;
    bool m_hasEmptyMAC;
    QString m_defaultSmbUser;
    QStringList m_badPaths;
    bool m_autoDetectBadPaths;
    NetworkPlaceListV2 m_networkPlaces;
};

} // namespace FlyFM

#endif // FLYFM_VFSSETTINGS_H
